@@ -1,7 +1,7 @@
 /*
  *  skype.c - Skype plugin for BitlBee
  *
- *  Copyright (c) 2007, 2008, 2009 by Miklos Vajna <vmiklos@frugalware.org>
+ *  Copyright (c) 2007, 2008, 2009, 2010 by Miklos Vajna <vmiklos@frugalware.org>
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -23,13 +23,20 @@
 #define _BSD_SOURCE
 #include <poll.h>
 #include <bitlbee.h>
-#include <bitlbee/ssl_client.h>
+#include <ssl_client.h>
 
 #define SKYPE_DEFAULT_SERVER "localhost"
 #define SKYPE_DEFAULT_PORT "2727"
 #define IRC_LINE_SIZE 1024
 #define ARRAY_SIZE(x) (sizeof(x)/sizeof(x[0]))
 
+/*
+ * Compatibility with BitlBee 1.3+
+ */
+#if BITLBEE_VERSION_CODE < BITLBEE_VER(1, 3, 0)
+#define B_EV_IO_READ GAIM_INPUT_READ
+#endif
+
 /*
  * Enumerations
  */
@@ -144,6 +151,9 @@ int skype_write(struct im_connection *ic, char *buf, int len)
 	struct skype_data *sd = ic->proto_data;
 	struct pollfd pfd[1];
 
+	if (!sd->ssl)
+		return FALSE;
+
 	pfd[0].fd = sd->fd;
 	pfd[0].events = POLLOUT;
 
@@ -234,6 +244,7 @@ void skype_call_ask(struct im_connection *ic, char *call_id, char *message)
 }
 struct groupchat *skype_chat_by_name(struct im_connection *ic, char *name)
 {
+#if BITLBEE_VERSION_CODE < BITLBEE_VER(1, 3, 0)
 	struct groupchat *ret;
 
 	for (ret = ic->groupchats; ret; ret = ret->next)
@@ -241,6 +252,9 @@ struct groupchat *skype_chat_by_name(struct im_connection *ic, char *name)
 			break;
 
 	return ret;
+#else
+	return bee_chat_by_title(ic->bee, ic, name);
+#endif
 }
 
 static char *skype_call_strerror(int err)
@@ -299,9 +313,12 @@ static void skype_parse_user(struct im_connection *ic, char *line)
 		return;
 	*ptr = '\0';
 	ptr++;
-	if (!strncmp(ptr, "ONLINESTATUS ", 13) &&
-			strcmp(user, sd->username) != 0
-			&& strcmp(user, "echo123") != 0) {
+	if (!strncmp(ptr, "ONLINESTATUS ", 13)) {
+			if (!strcmp(user, sd->username))
+				return;
+			if (!set_getbool(&ic->acc->set, "test_join")
+				&& !strcmp(user, "echo123"))
+				return;
 		ptr = g_strdup_printf("%s@skype.com", user);
 		imcb_add_buddy(ic, ptr, NULL);
 		if (strcmp(status, "OFFLINE") && (strcmp(status, "SKYPEOUT") ||
@@ -322,6 +339,31 @@ static void skype_parse_user(struct im_connection *ic, char *line)
 			imcb_add_buddy(ic, buf, NULL);
 			g_free(buf);
 		}
+	} else if (!strncmp(ptr, "MOOD_TEXT ", 10)) {
+		char *buf = g_strdup_printf("%s@skype.com", user);
+#if BITLBEE_VERSION_CODE < BITLBEE_VER(1, 3, 0)
+		user_t *u = user_findhandle(ic, buf);
+		g_free(buf);
+		buf = ptr + 10;
+		if (u) {
+			if (u->status_msg)
+				g_free(u->status_msg);
+			if (strlen(buf))
+				u->status_msg = g_strdup(buf);
+			else
+				u->status_msg = NULL;
+		}
+#else
+		bee_user_t *bu = bee_user_by_handle(ic->bee, ic, buf);
+		g_free(buf);
+		buf = ptr + 10;
+		if (bu) {
+			imcb_buddy_status(ic, bu->handle, bu->flags, NULL,
+					*buf ? buf : NULL);
+		}
+#endif
+		if (set_getbool(&ic->acc->set, "show_moods"))
+			imcb_log(ic, "User `%s' changed mood text to `%s'", user, buf);
 	} else if (!strncmp(ptr, "FULLNAME ", 9))
 		sd->info_fullname = g_strdup(ptr + 9);
 	else if (!strncmp(ptr, "PHONE_HOME ", 11))
@@ -749,11 +791,13 @@ static void skype_parse_chat(struct im_connection *ic, char *line)
 	if (gc)
 		imcb_chat_free(gc);
 	if (!strcmp(info, "STATUS MULTI_SUBSCRIBED")) {
-		imcb_chat_new(ic, id);
+		gc = imcb_chat_new(ic, id);
+		imcb_chat_name_hint(gc, id);
 		skype_printf(ic, "GET CHAT %s ADDER\n", id);
 		skype_printf(ic, "GET CHAT %s TOPIC\n", id);
 	} else if (!strcmp(info, "STATUS DIALOG") && sd->groupchat_with) {
 		gc = imcb_chat_new(ic, id);
+		imcb_chat_name_hint(gc, id);
 		/* According to the docs this
 		 * is necessary. However it
 		 * does not seem the situation
@@ -831,6 +875,8 @@ static void skype_parse_profile(struct im_connection *ic, char *line)
 
 static void skype_parse_ping(struct im_connection *ic, char *line)
 {
+	/* Unused parameter */
+	line = line;
 	skype_printf(ic, "PONG\n");
 }
 
@@ -874,6 +920,10 @@ static gboolean skype_read_callback(gpointer data, gint fd,
 		{ "CHATS ", skype_parse_chats },
 	};
 
+	/* Unused parameters */
+	fd = fd;
+	cond = cond;
+
 	if (!sd || sd->fd == -1)
 		return FALSE;
 	/* Read the whole data. */
@@ -917,7 +967,7 @@ gboolean skype_start_stream(struct im_connection *ic)
 		return FALSE;
 
 	if (sd->bfd <= 0)
-		sd->bfd = b_input_add(sd->fd, GAIM_INPUT_READ,
+		sd->bfd = b_input_add(sd->fd, B_EV_IO_READ,
 			skype_read_callback, ic);
 
 	/* Log in */
@@ -938,6 +988,10 @@ gboolean skype_connected(gpointer data, void *source, b_input_condition cond)
 {
 	struct im_connection *ic = data;
 	struct skype_data *sd = ic->proto_data;
+
+	/* Unused parameter */
+	cond = cond;
+
 	if (!source) {
 		sd->ssl = NULL;
 		imcb_error(ic, "Could not connect to server");
@@ -945,6 +999,7 @@ gboolean skype_connected(gpointer data, void *source, b_input_condition cond)
 		return FALSE;
 	}
 	imcb_log(ic, "Connected to server, logging in");
+
 	return skype_start_stream(ic);
 }
 
@@ -985,6 +1040,9 @@ static int skype_buddy_msg(struct im_connection *ic, char *who, char *message,
 	char *ptr, *nick;
 	int st;
 
+	/* Unused parameter */
+	flags = flags;
+
 	nick = g_strdup(who);
 	ptr = strchr(nick, '@');
 	if (ptr)
@@ -1015,6 +1073,9 @@ static void skype_set_away(struct im_connection *ic, char *state_txt,
 {
 	const struct skype_away_state *state;
 
+	/* Unused parameter */
+	message = message;
+
 	if (state_txt == NULL)
 		state = skype_away_state_by_name("Online");
 	else
@@ -1027,6 +1088,9 @@ static GList *skype_away_states(struct im_connection *ic)
 	static GList *l;
 	int i;
 
+	/* Unused parameter */
+	ic = ic;
+
 	if (l == NULL)
 		for (i = 0; skype_away_state_list[i].full_name; i++)
 			l = g_list_append(l,
@@ -1061,12 +1125,15 @@ static char *skype_set_call(set_t *set, char *value)
 	char *nick, *ptr;
 
 	if (value) {
+#if BITLBEE_VERSION_CODE < BITLBEE_VER(1, 3, 0)
+		/* IRC stuff shouldn't be touched from inside IM modules. */
 		user_t *u = user_find(acc->irc, value);
 		/* We are starting a call */
-		if (!u)
-			nick = g_strdup(value);
-		else
+		if (u)
 			nick = g_strdup(u->handle);
+		else
+#endif
+			nick = g_strdup(value);
 		ptr = strchr(nick, '@');
 		if (ptr)
 			*ptr = '\0';
@@ -1090,6 +1157,9 @@ static void skype_add_buddy(struct im_connection *ic, char *who, char *group)
 {
 	char *nick, *ptr;
 
+	/* Unused parameter */
+	group = group;
+
 	nick = g_strdup(who);
 	ptr = strchr(nick, '@');
 	if (ptr)
@@ -1103,6 +1173,9 @@ static void skype_remove_buddy(struct im_connection *ic, char *who, char *group)
 {
 	char *nick, *ptr;
 
+	/* Unused parameter */
+	group = group;
+
 	nick = g_strdup(who);
 	ptr = strchr(nick, '@');
 	if (ptr)
@@ -1114,6 +1187,10 @@ static void skype_remove_buddy(struct im_connection *ic, char *who, char *group)
 void skype_chat_msg(struct groupchat *gc, char *message, int flags)
 {
 	struct im_connection *ic = gc->ic;
+
+	/* Unused parameter */
+	flags = flags;
+
 	skype_printf(ic, "CHATMESSAGE %s %s\n", gc->title, message);
 }
 
@@ -1128,6 +1205,10 @@ void skype_chat_invite(struct groupchat *gc, char *who, char *message)
 {
 	struct im_connection *ic = gc->ic;
 	char *ptr, *nick;
+
+	/* Unused parameter */
+	who = who;
+
 	nick = g_strdup(message);
 	ptr = strchr(nick, '@');
 	if (ptr)
@@ -1223,6 +1304,11 @@ static void skype_init(account_t *acc)
 	s = set_add(&acc->set, "auto_join", "false", set_eval_bool, acc);
 	s->flags |= ACC_SET_OFFLINE_ONLY;
 
+	s = set_add(&acc->set, "test_join", "false", set_eval_bool, acc);
+	s->flags |= ACC_SET_OFFLINE_ONLY;
+
+	s = set_add(&acc->set, "show_moods", "false", set_eval_bool, acc);
+
 	s = set_add(&acc->set, "edit_prefix", "EDIT:",
 			NULL, acc);
 }
